// Copyright (C) 2014, Ives Rey-Otero, CMLA ENS Cachan
// // <ives.rey-otero@cmla.ens-cachan.fr>
// //
// // Version 20150806 (August 6th, 2015)
// //
// //
// // This program is free software: you can use, modify and/or
// // redistribute it under the terms of the GNU General Public
// // License as published by the Free Software Foundation, either
// // version 3 of the License, or (at your option) any later
// // version. You should have received a copy of this license along
// // this program. If not, see <http://www.gnu.org/licenses/>.
//
//
//
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <math.h>

#include "lib_io_gaussconv.h"
#include "lib_gaussconv.h"


// Write a formatted message to standard error and abort the program, for
// example, fatal_error("Failed to open file %s for writing", filename);
void fatal_error(const char* format, ...)
{
    va_list args;
    va_start(args, format);
    fprintf(stderr, "Fatal error: ");
    vfprintf(stderr, format, args);
    fprintf(stderr, "\n");
    va_end(args);
    exit(EXIT_FAILURE);
}


void print_usage()
{
    fprintf(stderr, "DEMO semigroup Lindeberg convolution  \n");
    fprintf(stderr, " gaussconv_lindeberg in sigma gamma(laplacian scheme) N(iterations) out  \n");
}


float rmse(float* x1, float* x2, int l)
{
    float diff;
    float rmse = 0.0;
    for(int i = 0; i < l; i++){
        diff = x1[i]-x2[i];
        rmse += diff*diff;
    }
    rmse = sqrt( rmse / (float)l );
    return rmse;
}



int main(int argc, char **argv)
{
    float sigma;
    float g;
    int N;

    switch (argc){
        case 6:  
            sigma = atof(argv[2]);
            g = atof(argv[3]);
            N = atoi(argv[4]);
            break;
        default:
            print_usage();
            return -1;
    }
    int w, h;
    float* x = read_image_to_gray(argv[1], &w, &h);
    if(!x)
        fatal_error("File \"%s\" not found.", argv[1]);
    float* t  = (float*)malloc(w*h*sizeof(float));  //temporary
    float* y1 = (float*)malloc(w*h*sizeof(float)); //direct
    float* y2 = (float*)malloc(w*h*sizeof(float)); //iterated
    float* d  = (float*)malloc(w*h*sizeof(float));  //difference

    // direct
    gaussconv_lindeberg(x, y1, w, h, sqrt((float)N)*sigma, g);

    // iterated
    for(int j=0; j<w*h; j++)
        t[j] = x[j];
    for(int i=0; i<N;i++){
        gaussconv_lindeberg(t, y2, w, h, sigma, g);
        for(int j=0; j<w*h;j++)
            t[j] = y2[j];
    }

    // difference image
    for(int j=0; j<w*h; j++){
        d[j] = y1[j]-y2[j];
    }

    // output
    char name[FILENAME_MAX];

    sprintf(name, "%s_direct.png", argv[5]);
    printImage(y1, w, h, name);

    sprintf(name, "%s_iter.png", argv[5]);
    printImage(y2, w, h, name);

    sprintf(name, "%s_diff.png", argv[5]);
    printImage_LinearConversion(d, w, h, name);

    float err = rmse(y1, y2, w*h);
    fprintf(stdout, "RMSE = %g \n", err);
    
    free(x);
    free(t);
    free(y1);
    free(y2);
    free(d);

   
    return EXIT_SUCCESS;
}
