
// Copyright (C) 2014, Ives Rey-Otero, CMLA ENS Cachan
// // <ives.rey-otero@cmla.ens-cachan.fr>
// //
// // Version 20150806 (August 6th, 2015)
// //
// //
// // This program is free software: you can use, modify and/or
// // redistribute it under the terms of the GNU General Public
// // License as published by the Free Software Foundation, either
// // version 3 of the License, or (at your option) any later
// // version. You should have received a copy of this license along
// // this program. If not, see <http://www.gnu.org/licenses/>.
//
//
//


#include <stdlib.h>
#include <stdio.h>
#include <float.h>

#include "io_png.h"

#include "lib_io_gaussconv.h"
#define MAX(i,j) ( (i)<(j) ? (j):(i) )
#define MIN(i,j) ( (i)<(j) ? (i):(j) )



// Allocate memory or abort on failure
static void* xmalloc(size_t size)
{
    if (size == 0)
        fprintf(stderr,"xmalloc: zero size");
    void *p = malloc(size);
    if (!p)
    {
        float sm = size / (0x100000 * 1.0);
        fprintf(stderr,"xmalloc: out of memory when requesting "
                "%zu bytes (%gMB)",//:\"%s\"",
                size, sm);//, strerror(errno));
    }
    return p;
}


// Free memory allocated by xmalloc.
static void xfree(void* p)
{
    if (!p)
        fprintf(stderr,"trying to free a null pointer");
    free(p);
}





float* read_image_to_gray(char* fname, int *pw, int *ph)
{
    size_t w, h;
    float* imf32 = io_png_read_f32_rgb(fname, &w, &h);
    float* x = xmalloc(w*h*sizeof(float));
    for(int i = 0; i < (int)(w*h); i++)
        x[i] = (float)((0.299*imf32[i] + 0.587*imf32[w*h+i] + 0.1140*imf32[2*w*h+i]))/255.0; //RGB2GRAY
    xfree(imf32);
    *pw = w;
    *ph = h;
    return x;
}




void printImage(float *imageIn, int w, int h, char *name)
{
    float *imf32 = xmalloc(w * h * sizeof(float));
    for (int i = 0; i < w * h; i++) {
        imf32[i] =  (float)(255.* imageIn[i]);
    }
    io_png_write_f32(name, imf32, w, h, 1);
    xfree(imf32);
}



void linear_conversion(float *imIn, float *imOut, int length){
    float a, b;
    float minVal = DBL_MAX;
    float maxVal = -DBL_MAX;
    for (int i = 0; i < length; i++) {
        if (imIn[i] >= maxVal){
            maxVal = imIn[i];
        }
        if (imIn[i] <= minVal){
            minVal = imIn[i];
        }
    }
    a = (0.9 - 0.0) / (maxVal - minVal);
    b = -a * minVal;
    for (int i = 0; i < length; i++)
        imOut[i] = a * imIn[i] + b;
}

void printImage_LinearConversion(float *imageIn, int w, int h, char *name){
    float *imTemp = xmalloc(w*h*sizeof(float));
    linear_conversion(imageIn, imTemp, w * h);
    printImage(imTemp, w, h, name);
    xfree(imTemp);
}


